@@ -230,14 +230,6 @@ namespace Interface
             const int32_t objectType = getSelectedObjectType();
             if ( objectType >= 0 ) {
                 const Maps::ObjectGroup objectGroup = getSelectedObjectGroup();
-                if ( objectGroup == Maps::ObjectGroup::ADVENTURE_MINES ) {
-                    // For mine we need to decode the objectType.
-                    int32_t type = -1;
-                    int32_t color = -1;
-                    getMineObjectProperties( type, color );
-
-                    return getObjectOccupiedArea( objectGroup, type );
-                }
 
                 return getObjectOccupiedArea( objectGroup, objectType );
             }
@@ -801,11 +793,11 @@ namespace Interface
             return;
         case Instrument::LANDSCAPE_OBJECTS:
             switch ( _selectedLandscapeObject ) {
+            case LandscapeObjectBrush::LANDSCAPE_MISC:
             case LandscapeObjectBrush::MOUNTAINS:
             case LandscapeObjectBrush::ROCKS:
             case LandscapeObjectBrush::TREES:
             case LandscapeObjectBrush::WATER_OBJECTS:
-            case LandscapeObjectBrush::LANDSCAPE_MISC:
                 _interface.setCursorUpdater(
                     [type = getSelectedObjectType(), group = getSelectedObjectGroup()]( const int32_t /*tileIndex*/ ) { setCustomCursor( group, type ); } );
                 return;
@@ -815,35 +807,16 @@ namespace Interface
             break;
         case Instrument::ADVENTURE_OBJECTS:
             switch ( _selectedAdventureObject ) {
+            case AdventureObjectBrush::ADVENTURE_MISC:
             case AdventureObjectBrush::ARTIFACTS:
             case AdventureObjectBrush::DWELLINGS:
+            case AdventureObjectBrush::MINES:
             case AdventureObjectBrush::POWER_UPS:
             case AdventureObjectBrush::TREASURES:
             case AdventureObjectBrush::WATER_ADVENTURE:
-            case AdventureObjectBrush::ADVENTURE_MISC:
                 _interface.setCursorUpdater(
                     [type = getSelectedObjectType(), group = getSelectedObjectGroup()]( const int32_t /*tileIndex*/ ) { setCustomCursor( group, type ); } );
                 return;
-            case AdventureObjectBrush::MINES:
-                _interface.setCursorUpdater( [this]( const int32_t /*tileIndex*/ ) {
-                    int32_t type = -1;
-                    int32_t color = -1;
-                    getMineObjectProperties( type, color );
-
-                    if ( type == -1 || color == -1 ) {
-                        // The object type is not set. We show the POINTER cursor for this case.
-                        Cursor::Get().SetThemes( Cursor::POINTER );
-                        return;
-                    }
-
-                    assert( Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES ).size() > static_cast<size_t>( type ) );
-
-                    // TODO: Implement a function to render also the owner flag after ownership selection is implemented.
-                    const fheroes2::Sprite & image = getObjectImage( Maps::ObjectGroup::ADVENTURE_MINES, type );
-
-                    Cursor::Get().setCustomImage( image, { image.x(), image.y() } );
-                } );
-                return;
             default:
                 break;
             }
@@ -1113,15 +1086,7 @@ namespace Interface
                 return res;
             }
             if ( le.MouseClickLeft( _adventureObjectButtonsRect[AdventureObjectBrush::MINES] ) ) {
-                handleObjectMouseClick( [this]( const int32_t /* type */ ) {
-                    int32_t type = -1;
-                    int32_t color = -1;
-
-                    getMineObjectProperties( type, color );
-                    Dialog::selectMineType( type, color );
-
-                    return _generateMineObjectProperties( type, color );
-                } );
+                handleObjectMouseClick( Dialog::selectMineType );
                 return res;
             }
             if ( le.MouseClickLeft( _adventureObjectButtonsRect[AdventureObjectBrush::DWELLINGS] ) ) {
@@ -1206,12 +1171,12 @@ namespace Interface
             }
         }
 
-        _buttonMagnify.drawOnState( le.isMouseLeftButtonPressedInArea( _rectMagnify ) );
-        _buttonUndo.drawOnState( le.isMouseLeftButtonPressedInArea( _rectUndo ) );
-        _buttonRedo.drawOnState( le.isMouseLeftButtonPressedInArea( _rectRedo ) );
-        _buttonSpecs.drawOnState( le.isMouseLeftButtonPressedInArea( _rectSpecs ) );
-        _buttonFile.drawOnState( le.isMouseLeftButtonPressedInArea( _rectFile ) );
-        _buttonSystem.drawOnState( le.isMouseLeftButtonPressedInArea( _rectSystem ) );
+        _buttonMagnify.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectMagnify ) );
+        _buttonUndo.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectUndo ) );
+        _buttonRedo.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectRedo ) );
+        _buttonSpecs.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectSpecs ) );
+        _buttonFile.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectFile ) );
+        _buttonSystem.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectSystem ) );
 
         if ( le.MouseClickLeft( _rectMagnify ) ) {
             _interface.eventViewWorld();
@@ -1335,32 +1300,4 @@ namespace Interface
 
         return color * static_cast<int32_t>( townObjects.size() ) + type;
     }
-
-    void EditorPanel::getMineObjectProperties( int32_t & type, int32_t & color ) const
-    {
-        const auto & mineObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES );
-        if ( mineObjects.empty() ) {
-            // How is it even possible?
-            assert( 0 );
-            type = -1;
-            color = -1;
-            return;
-        }
-
-        type = _selectedAdventureObjectType[AdventureObjectBrush::MINES] % static_cast<int32_t>( mineObjects.size() );
-        color = _selectedAdventureObjectType[AdventureObjectBrush::MINES] / static_cast<int32_t>( mineObjects.size() );
-    }
-
-    int32_t EditorPanel::_generateMineObjectProperties( const int32_t type, const int32_t color )
-    {
-        const auto & mineObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES );
-        if ( mineObjects.empty() ) {
-            // How is it even possible?
-            assert( 0 );
-            return -1;
-        }
-
-        const int32_t objectType = ( color * static_cast<int32_t>( mineObjects.size() ) + type );
-        return ( objectType < 0 ) ? -1 : objectType;
-    }
 }
